from pygments import highlight
from typing import Optional, Union
from tabulate import tabulate
from functools import partial
import pandas as pd
from better_highlighting.json_lexer import JSONLexer
from pygments.lexers import PythonLexer
from pygments.formatters import Terminal256Formatter
from better_highlighting.style_builders import SimpleStyle, JSONStyle
from better_highlighting.data_format import pretty_as_iterator, pretty_as_text, make_it_short


class __Printer:

    def __init__(self):
        self._font: str = str()
        self._colors: str = str()

    def display(self, text, colors_style: Optional[str] = None,
                font_style: Optional[str] = None,
                pretty_view=False,
                short=False):
        self._font = font_style or str()
        self._colors = colors_style or str()

        text_to_process = make_it_short(text) if short else text

        if pretty_view:
            string_to_color = pretty_as_iterator(text_to_process)
        else:
            string_to_color = pretty_as_text(text_to_process)
        try:
            s = self._style
            h = highlight(
                string_to_color,
                JSONLexer(),
                Terminal256Formatter(style=s)
            )
            print(h)
        except AttributeError as e:
            raise e from e

    def store(self, text, colors_style: Optional[str] = None,
              font_style: Optional[str] = None,
              pretty_view=False,
              short=False) -> str:

        self._font = font_style or str()
        self._colors = colors_style or str()

        text_to_process = make_it_short(text) if short else text

        if pretty_view:
            string_to_color = pretty_as_iterator(text_to_process)
        else:
            string_to_color = pretty_as_text(text_to_process)
        try:
            return highlight(
                string_to_color,
                JSONLexer(),
                Terminal256Formatter(style=self._style)
            )
        except AttributeError as e:
            raise e from e

    def table(self, data: Union[list, str, dict],
              colors_style: Optional[str] = None,
              short=False, transpose=False, headers=True, store=False):

        data_to_process = (make_it_short(data, nested=True) if short else data) \
            if isinstance(data, list) \
            else [make_it_short(data, nested=True) if short else data]

        df = pd.DataFrame(data_to_process)

        df.columns = [f'||{str(column).upper()}||' for column in df.columns]

        for c in df.columns:
            df[c] = df[c].apply(partial(pretty_as_iterator, htchar=' '))

        if transpose:
            table_opt = {'tabular_data': df.transpose(),
                         'showindex': headers,
                         'tablefmt': "fancy_grid",
                         'colalign': ("left",)
                         }
        else:
            table_opt = {'tabular_data': df,
                         'showindex': False,
                         'tablefmt': "fancy_grid",
                         'colalign': ("left",)
                         }

            table_opt.update({'headers': df.columns}) if headers else ...

        highlighted = highlight(tabulate(**table_opt),
                                JSONLexer(),
                                Terminal256Formatter(style=JSONStyle(colors_style).style_obj)).replace('||', '  ')
        if store:
            return highlighted
        else:
            print(highlighted)

    @property
    def _style(self):
        if self._font == 'simple':
            self._font = str()
        if self._colors == 'json':
            return JSONStyle().style_obj
        style_cfg = f'{self._font} {self._colors}'
        return SimpleStyle(style_cfg).style_obj


printer = __Printer()

# if __name__ == '__main__':
#     font_styles = (
#         'simple',
#         'bold',
#         'italic',
#         'italic bold',
#         'underline',
#         'italic underline',
#         'bold underline',
#         'italic bold underline')
#
#     colors = (
#         # dark
#         'ansiblack',
#         'ansired',
#         'ansigreen',
#         'ansiyellow',
#         'ansiblue',
#         'ansimagenta',
#         'ansicyan',
#         'ansigray',
#         # normal
#         'ansibrightblack',
#         'ansibrightred',
#         'ansibrightgreen',
#         'ansibrightyellow',
#         'ansibrightblue',
#         'ansibrightmagenta',
#         'ansibrightcyan',
#         'ansiwhite')
#
#     demo_list = ['list_element_1', 'list_element_2', 'list_element_3', 'list_element_4']
#     demo_tuple = ('tuple _element_1', 'tuple_element_2', 'tuple_element_3', 'tuple_element_4')
#     demo_dict = {'Header_1': demo_list, 'HEADER_2': demo_tuple, 'h_e_a_d_e_r': 'some text', 1234567: 5555888999}
#
#     print(' '.join([printer.store('style', font_style=font_style).strip('\n') for font_style in font_styles]))
#
#     print(' '.join([printer.store('color', font_style=color).strip('\n') for color in colors]))
#
#     printer.display(demo_list, pretty_view=True)
#     printer.display(demo_list, colors_style='ansicyan')
#     printer.display(demo_list, colors_style='json')
#     printer.display(demo_tuple, pretty_view=True, colors_style='json')
#     printer.display(demo_tuple, colors_style='ansiyellow')
#     printer.display(demo_dict, colors_style='json')
#     printer.display(demo_dict, pretty_view=True, colors_style='json')
#
#     printer.table(demo_dict)
#     printer.table(demo_dict, transpose=True)



import re

from pygments.lexer import RegexLexer, include, bygroups, \
    default, combined
from pygments.util import shebang_matches
from pygments.token import Text, Comment, Operator, Keyword, Name, String, \
    Number, Punctuation, Token
from pygments import unistring as uni

CustomTrue = Token.CustomTrue
CustomFalse = Token.CustomFalse
CustomNull = Token.Null
FormatSeparator = Token.FormatSeparator
SpecialMessages = Token.SpecialMessages
Table_1 = Token.Table_1
Table_Header = Token.Header


class JSONPythonLexer(RegexLexer):

    name = 'JSONPython'

    uni_name = "[%s][%s]*" % (uni.xid_start, uni.xid_continue)

    def innerstring_rules(ttype):
        return [
            # the old style '%s' % (...) string formatting (still valid in Py3)
            (r'%(\(\w+\))?[-#0 +]*([0-9]+|[*])?(\.([0-9]+|[*]))?'
             '[hlL]?[E-GXc-giorsaux%]', String.Interpol),
            # the new style '{}'.format(...) string formatting
            (r'\{'
             r'((\w+)((\.\w+)|(\[[^\]]+\]))*)?'  # field name
             r'(\![sra])?'  # conversion
             r'(\:(.?[<>=\^])?[-+ ]?#?0?(\d+)?,?(\.\d+)?[E-GXb-gnosx%]?)?'
             r'\}', String.Interpol),

            # backslashes, quotes and formatting signs must be parsed one at a time
            (r'[^\\\'"%{\n]+', ttype),
            (r'[\'"\\]', ttype),
            # unhandled string formatting sign
            (r'%|(\{{1,2})', ttype)
            # newlines are an error (use "nl" state)
        ]

    # flags = re.MULTILINE | re.UNICODE

    tokens = {
        'root': [

            (r'(?<=\|\|)([a-zA-Z1-9 _]*)(?=\|\|)', Table_Header),

            (r'\n', Text),
            (r'^(\s*)([rRuUbB]{,2})("""(?:.|\n)*?""")',
             bygroups(Text, String.Affix, String.Doc)),
            (r"^(\s*)([rRuUbB]{,2})('''(?:.|\n)*?''')",
             bygroups(Text, String.Affix, String.Doc)),
            (r'\A#!.+$', Comment.Hashbang),
            (r'#.*$', Comment.Single),
            (r'\\\n', Text),
            (r'\\', Text),
            # (r'(def)((?:\s|\\\s)+)', bygroups(Keyword, Text), 'funcname'),
            # (r'(class)((?:\s|\\\s)+)', bygroups(Keyword, Text), 'classname'),

            (r'several items were not printed', SpecialMessages),

            (r'True|true', CustomTrue),
            (r'False|false', CustomFalse),
            (r'None|none', CustomNull),
            #
            # (r'[a-zA-Z_]\w*', Name.Addition_1),
            (r'[1-9]+[a-z]', Text.Addition_2),
            (r'[?!]', Text.Addition_3),

            (r'(?<=\n\t.)\+(?=[a-zA-Z1-9])', FormatSeparator),
            (r'[╒═╤╕╞╪╡├─┼┤╘╧╛│]', Table_1),

            include('expr'),


        ],
        'expr': [
            # raw f-strings
            ('(?i)(rf|fr)(""")',
             bygroups(String.Affix, String.Double),
             combined('rfstringescape', 'tdqf')),
            ("(?i)(rf|fr)(''')",
             bygroups(String.Affix, String.Single),
             combined('rfstringescape', 'tsqf')),
            ('(?i)(rf|fr)(")',
             bygroups(String.Affix, String.Double),
             combined('rfstringescape', 'dqf')),
            ("(?i)(rf|fr)(')",
             bygroups(String.Affix, String.Single),
             combined('rfstringescape', 'sqf')),
            # non-raw f-strings
            ('([fF])(""")', bygroups(String.Affix, String.Double),
             combined('fstringescape', 'tdqf')),
            ("([fF])(''')", bygroups(String.Affix, String.Single),
             combined('fstringescape', 'tsqf')),
            ('([fF])(")', bygroups(String.Affix, String.Double),
             combined('fstringescape', 'dqf')),
            ("([fF])(')", bygroups(String.Affix, String.Single),
             combined('fstringescape', 'sqf')),
            # raw strings
            ('(?i)(rb|br|r)(""")',
             bygroups(String.Affix, String.Double), 'tdqs'),
            ("(?i)(rb|br|r)(''')",
             bygroups(String.Affix, String.Single), 'tsqs'),
            ('(?i)(rb|br|r)(")',
             bygroups(String.Affix, String.Double), 'dqs'),
            ("(?i)(rb|br|r)(')",
             bygroups(String.Affix, String.Single), 'sqs'),
            # non-raw strings
            ('([uUbB]?)(""")', bygroups(String.Affix, String.Double),
             combined('stringescape', 'tdqs')),
            ("([uUbB]?)(''')", bygroups(String.Affix, String.Single),
             combined('stringescape', 'tsqs')),
            ('([uUbB]?)(")', bygroups(String.Affix, String.Double),
             combined('stringescape', 'dqs')),
            ("([uUbB]?)(')", bygroups(String.Affix, String.Single),
             combined('stringescape', 'sqs')),
            (r'[^\S\n]+', Text),
            include('numbers'),
            (r'!=|==|<<|>>|:=|[-~+/*%=<>&^|.]', Operator),
            (r'[]{}:(),;[]', Punctuation),
            (r'(in|is|and|or|not)\b', Operator.Word),
            include('name'),
        ],
        'numbers': [
            (r'(\d(?:_?\d)*\.(?:\d(?:_?\d)*)?|(?:\d(?:_?\d)*)?\.\d(?:_?\d)*)'
             r'([eE][+-]?\d(?:_?\d)*)?', Number.Float),
            (r'\d(?:_?\d)*[eE][+-]?\d(?:_?\d)*j?', Number.Float),
            (r'0[oO](?:_?[0-7])+', Number.Oct),
            (r'0[bB](?:_?[01])+', Number.Bin),
            (r'0[xX](?:_?[a-fA-F0-9])+', Number.Hex),
            (r'\d(?:_?\d)*', Number.Integer),
        ],
        'name': [
            (r'@' + uni_name, Name.Decorator),
            (r'@', Operator),  # new matrix multiplication operator
            (uni_name, Name),
        ],
        # 'funcname': [
        #     (uni_name, Name.Function, '#pop'),
        #     default('#pop'),
        # ],
        # 'classname': [
        #     (uni_name, Name.Class, '#pop'),
        # ],
        'rfstringescape': [
            (r'\{\{', String.Escape),
            (r'\}\}', String.Escape),
        ],
        'fstringescape': [
            include('rfstringescape'),
            include('stringescape'),
        ],
        'stringescape': [
            (r'\\([\\abfnrtv"\']|\n|N\{.*?\}|u[a-fA-F0-9]{4}|'
             r'U[a-fA-F0-9]{8}|x[a-fA-F0-9]{2}|[0-7]{1,3})', String.Escape)
        ],
        'strings-single': innerstring_rules(String.Single),
        'strings-double': innerstring_rules(String.Double),
        'dqf': [
            (r'"', String.Double, '#pop'),
            (r'\\\\|\\"|\\\n', String.Escape),  # included here for raw strings
        ],
        'sqf': [
            (r"'", String.Single, '#pop'),
            (r"\\\\|\\'|\\\n", String.Escape),  # included here for raw strings
        ],
        'dqs': [
            (r'"', String.Double, '#pop'),
            (r'\\\\|\\"|\\\n', String.Escape),  # included here for raw strings
            include('strings-double')
        ],
        'sqs': [
            (r"'", String.Single, '#pop'),
            (r"\\\\|\\'|\\\n", String.Escape),  # included here for raw strings
            include('strings-single')
        ],
        'tdqf': [
            (r'"""', String.Double, '#pop'),
            (r'\n', String.Double)
        ],
        'tsqf': [
            (r"'''", String.Single, '#pop'),
            (r'\n', String.Single)
        ],
        'tdqs': [
            (r'"""', String.Double, '#pop'),
            include('strings-double'),
            (r'\n', String.Double)
        ],
        'tsqs': [
            (r"'''", String.Single, '#pop'),
            include('strings-single'),
            (r'\n', String.Single)
        ],
    }

    def analyse_text(text):
        return shebang_matches(text, r'pythonw?(3(\.\d)?)?') or \
               'import ' in text[:1000]


JSONLexer = JSONPythonLexer



from pygments.token import Token
from pygments.style import StyleMeta

_tokens_simple = (Token.Text, Token.Name, Token.Punctuation, Token.Operator, Token.Number, Token.String, Token.Keyword,
                  Token.Generic, Token.Whitespace, Token.Escape, Token.Other, Token.AdditionalSymbols, Token.Table_1,
                  Token.CustomTrue, Token.CustomFalse, Token.FormatSeparator, Token.Null)


class SimpleStyle:

    def __init__(self, font_style: str):
        self.style_obj = StyleMeta('SimpleStyle', (),
                                   {'__module__': self.__module__,
                                    '__qualname__': 'Style',
                                    # 'background_color': '#ffffff',
                                    # 'highlight_color': '#ffffcc',
                                    # 'line_number_color': 'inherit',
                                    # 'line_number_background_color': 'transparent',
                                    # 'line_number_special_color': '#000000',
                                    # 'line_number_special_background_color': '#ffffc0',
                                    'styles': {token: font_style for token in _tokens_simple},
                                    # 'web_style_gallery_exclude': False
                                    })


class JSONStyle:
    styles = {
        Token.Punctuation: 'ansired',
        Token.String: '#ffa500',
        Token.Name: "ansigreen",#'#ffa500',
        Token.SpecialMessages: 'italic #76756C',
        Token.CustomTrue: 'bold #42EB53',
        Token.CustomFalse: 'bold #FF001A',
        Token.FormatSeparator: '#FFFFFF',
        Token.Text: 'ansigreen',
        Token.Null: '#40ffff',
        Token.Number: '#3677a9'
    }

    def __init__(self, color=None):
        self.styles.update({Token.Table_1: color}) if color else self.styles.update({Token.Table_1: ''})

        self.style_obj = StyleMeta('JSONStyle', (),
                                   {'__module__': self.__module__,
                                    '__qualname__': 'Style',
                                    # 'background_color': '#ffffff',
                                    # 'highlight_color': '#ffffcc',
                                    # 'line_number_color': 'inherit',
                                    # 'line_number_background_color': 'transparent',
                                    # 'line_number_special_color': '#000000',
                                    # 'line_number_special_background_color': '#ffffc0',
                                    'styles': self.styles,
                                    # 'web_style_gallery_exclude': False
                                    })



import textwrap as tw
from typing import Union, Optional
from dataclasses import is_dataclass, asdict, dataclass

MIN_LENGTH_STR = 250
MIN_LENGTH_LIST = 3
MIN_LENGTH_DICT = 10
MIN_LENGTH_STR_IN_DICT = 40


def pretty_as_iterator(value, htchar='\t', lfchar='\n', indent=0, key_length=0) -> str:
    tab_key_length = 4 if key_length != 0 else 0
    nlch = lfchar + htchar * (indent + 1)
    if type(value) is dict:
        items = [
            nlch + repr(key) + ': ' + pretty_as_iterator(value[key], htchar, lfchar, indent + 1,
                                                         key_length=len(str(key)))
            for key in value
        ]
        return '{%s}' % (','.join(items) + lfchar + htchar * indent)
    elif type(value) is list:
        items = [
            nlch + pretty_as_iterator(item, htchar, lfchar, indent + 1, key_length=0)
            for item in value
        ]
        return f"[{','.join(items) + lfchar + htchar * indent}]"
    elif type(value) is tuple:
        items = [
            nlch + pretty_as_iterator(item, htchar, lfchar, indent + 1, key_length=0)
            for item in value
        ]
        return f"({','.join(items) + lfchar + htchar * indent})"
    elif type(value) is str:
        w = MIN_LENGTH_STR_IN_DICT - (len(lfchar) + len(htchar) * (indent + (key_length + tab_key_length)))
        value = tw.wrap(value, width=w + 1)
        return f' {lfchar + htchar * (indent + key_length)}+'.join(value)
        # return f"{lfchar + htchar * (indent + (key_length + tab_key_length))}+".join(value) For table format
    elif type(value) is int:
        w = MIN_LENGTH_STR_IN_DICT - (len(lfchar) + len(htchar) * (indent + (key_length + tab_key_length)))
        value = tw.wrap(str(value), width=w + 1)
        return f' {lfchar + htchar * (indent + key_length)}+'.join(value)
    return repr(value)


def pretty_as_text(value) -> str:
    if type(value) is dict:
        items = [f'{repr(key)}: {pretty_as_text(value[key])}' for key in value]
        return '{%s}' % (', '.join(items))
    elif type(value) is list:
        items = [
            pretty_as_text(item)
            for item in value
        ]
        return f"[{', '.join(items)}]"
    elif type(value) is tuple:
        items = [
            pretty_as_text(item)
            for item in value
        ]
        return f"({', '.join(items)})"
    elif type(value) is str:
        value = tw.wrap(value)
        return f''.join(value)

    return repr(value)


def make_it_short(value: Union[str, list, dict, dataclass],
                  nested: Optional[bool] = False) -> Union[str, list, dict]:

    if type(value) is list:
        if (length := len(value)) > MIN_LENGTH_LIST:
            formatted_list = value.copy()
            del formatted_list[MIN_LENGTH_LIST:]
            if not nested:
                formatted_list.append(f'several items were not printed {length - MIN_LENGTH_LIST}')

            return [make_it_short(item) for item in formatted_list]
        return [make_it_short(item) for item in value]

    elif type(value) is dict:
        if (length := len(value)) > MIN_LENGTH_DICT:
            formatted_data = value.copy()
            keys = list(value.keys())
            del keys[:MIN_LENGTH_DICT]
            for key in keys:
                del formatted_data[key]

            formatted_data = {key: make_it_short(formatted_data[key]) for key in formatted_data}

            return (
                formatted_data
                if nested
                else [
                    formatted_data,
                    f'several items were not printed {length - MIN_LENGTH_DICT}',
                ]
            )

        return {key: make_it_short(value[key]) for key in value}

    if is_dataclass(value):
        return make_it_short(asdict(value))

    if type(value) is str:
        max_size = MIN_LENGTH_STR // 5
        if (length := len(value)) > MIN_LENGTH_STR:
            formatted_str = []
            i = 0
            for k in range(max_size, length, max_size):
                formatted_str.append(value[i:k])
                i += max_size
            formatted_data = make_it_short(formatted_str, nested=True)
            return f'{"".join(formatted_data)} ...'
        return value

    return value



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['better_highlighting']

package_data = \
{'': ['*']}

install_requires = \
['Pygments>=2.11.2,<3.0.0', 'pandas>=1.4.1,<2.0.0', 'tabulate>=0.8.9,<0.9.0']

setup_kwargs = {
    'name': 'better-highlighting',
    'version': '0.1.2',
    'description': 'Small tool for text and syntax highlight.',
    'long_description': '[![Python 3.9](https://img.shields.io/badge/python-3.9-blue.svg)](https://www.python.org/downloads/release/python-390/)\n\n# Better highlighting\n\n\n### Usage:\nThis tool is designed for beautiful text formatting in the terminal. It is highly specialized, prints in terminal highlighted text \nin more readable form. \nIt also can render \'pretty\' table and python\'s iterators syntax.\n\n### Example:\n<img src="https://user-images.githubusercontent.com/21011049/160372378-32acc15e-1cfa-4987-bce7-6dfd34ec3ab2.png"></img> \n\n### Text styles\n<table>\n<th>Styles</th>\n<tr><td>ansiblack</td></tr>\n<tr><td>bold</td></tr>\n<tr><td>italic</td></tr>\n<tr><td>underline</td></tr>\n</table>\n\n\n### Colors\nColors specified using ansi* are converted to a default set of RGB colors when used with formatters other than the terminal-256 formatter.\n\nBy definition of ANSI, the following colors are considered “light” colors, and will be rendered by most terminals as bold:\n\n* “brightblack” (darkgrey), “brightred”, “brightgreen”, “brightyellow”, “brightblue”, “brightmagenta”, “brightcyan”, “white”\n\nThe following are considered “dark” colors and will be rendered as non-bold:\n\n* “black”, “red”, “green”, “yellow”, “blue”, “magenta”, “cyan”, “gray”\n\n<table>\n<th>Color names</th>\n<tr><td>ansiblack</td></tr>\n<tr><td>ansired</td></tr>\n<tr><td>ansigreen</td></tr>\n<tr><td>ansiyellow</td></tr>\n<tr><td>ansiblue</td></tr>\n<tr><td>ansimagenta</td></tr>\n<tr><td>ansibrightblack</td></tr>\n<tr><td>ansibrightred</td></tr>\n<tr><td>ansibrightgreen</td></tr>\n<tr><td>ansibrightyellow</td></tr>\n<tr><td>ansibrightblue</td></tr>\n<tr><td>ansibrightmagenta</td></tr>\n<tr><td>ansibrightcyan</td></tr>\n<tr><td>ansiwhite</td></tr>\n</table>\n',
    'author': 'vskarinov',
    'author_email': 'skarinov27@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/vskarinov/better_highlighting',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
